@@ -17,7 +17,7 @@
 """Python bindings for Subversion."""
 
 __author__ = "Jelmer Vernooij <jelmer@samba.org>"
-__version__ = (0, 7, 3)
+__version__ = (0, 7, 5)
 
 NODE_DIR = 2
 NODE_FILE = 1
@@ -72,6 +72,10 @@ ERR_EAI_NONAME = 670008
 ERR_FS_CONFLICT = 160024
 ERR_NODE_UNKNOWN_KIND = 145000
 ERR_RA_SERF_SSL_CERT_UNTRUSTED = 230001
+ERR_ENTRY_NOT_FOUND = 150000
+
+ERR_APR_OS_START_EAIERR = 670000
+ERR_APR_OS_ERRSPACE_SIZE = 50000
 
 AUTH_PARAM_DEFAULT_USERNAME = 'svn:auth:username'
 AUTH_PARAM_DEFAULT_PASSWORD = 'svn:auth:password'
