@@ -15,17 +15,24 @@
 
 """Subversion ra library tests."""
 
-from unittest import TestCase
-
 from subvertpy import (
     wc,
     )
+from subvertpy.tests import (
+    TestCase,
+    )
 
 class VersionTest(TestCase):
 
     def test_version_length(self):
         self.assertEquals(4, len(wc.version()))
 
+    def test_api_version_length(self):
+        self.assertEquals(4, len(wc.api_version()))
+
+    def test_api_version_later_same(self):
+        self.assertTrue(wc.api_version() <= wc.version())
+
 
 class WorkingCopyTests(TestCase):
 
@@ -40,3 +47,12 @@ class WorkingCopyTests(TestCase):
 
     def test_is_wc_prop(self):
         self.assertTrue(wc.is_wc_prop("svn:wc:foo"))
+
+    def test_match_ignore_list(self):
+        if wc.api_version() < (1, 5):
+            self.assertRaises(NotImplementedError, wc.match_ignore_list, "foo", [])
+            return # Skip test
+        self.assertTrue(wc.match_ignore_list("foo", [ "f*"]))
+        self.assertTrue(wc.match_ignore_list("foo", ["foo"]))
+        self.assertFalse(wc.match_ignore_list("foo", []))
+        self.assertFalse(wc.match_ignore_list("foo", ["bar"]))
