@@ -65,10 +65,10 @@ PyTypeObject *PyErr_GetSubversionExceptionTypeObject(void)
 	if (coremod == NULL) {
 		return NULL;
 	}
-		
+
 	excobj = PyObject_GetAttrString(coremod, "SubversionException");
 	Py_DECREF(coremod);
-	
+
 	if (excobj == NULL) {
 		PyErr_BadInternalCall();
 		return NULL;
@@ -96,6 +96,7 @@ PyObject *PyErr_NewSubversionException(svn_error_t *error)
 		child = cls->tp_new(cls, args, NULL);
 		if (cls->tp_init != NULL)
 			cls->tp_init(child, args, NULL);
+		Py_DECREF(cls);
 		Py_DECREF(args);
 	} else {
 		child = Py_None;
@@ -116,13 +117,17 @@ void PyErr_SetSubversionException(svn_error_t *error)
 	PyObject *excval, *excobj;
 
 	if (error->apr_err < 1000) {
-		PyErr_SetObject(PyExc_OSError, Py_BuildValue("(iz)", error->apr_err, error->message));
+		PyObject *excval = Py_BuildValue("(iz)", error->apr_err, error->message);
+		PyErr_SetObject(PyExc_OSError, excval);
+		Py_DECREF(excval);
 		return;
 	}
-	
+
 	if (error->apr_err >= APR_OS_START_SYSERR && 
 		error->apr_err < APR_OS_START_SYSERR + 1000) {
-		PyErr_SetObject(PyExc_OSError, Py_BuildValue("(iz)", error->apr_err - APR_OS_START_SYSERR, error->message));
+		PyObject *excval = Py_BuildValue("(iz)", error->apr_err - APR_OS_START_SYSERR, error->message);
+		PyErr_SetObject(PyExc_OSError, excval);
+		Py_DECREF(excval);
 		return;
 	}
 
@@ -132,6 +137,8 @@ void PyErr_SetSubversionException(svn_error_t *error)
 
 	excval = PyErr_NewSubversionException(error);
 	PyErr_SetObject(excobj, excval);
+	Py_DECREF(excval);
+	Py_DECREF(excobj);
 }
 
 PyObject *PyOS_tmpfile(void)
@@ -236,7 +243,7 @@ PyObject *prop_hash_to_dict(apr_hash_t *props)
 	apr_pool_t *pool;
 	PyObject *py_props;
 	if (props == NULL) {
-		Py_RETURN_NONE;
+		return PyDict_New();
 	}
 	pool = Pool(NULL);
 	if (pool == NULL)
@@ -299,7 +306,7 @@ apr_hash_t *prop_dict_to_hash(apr_pool_t *pool, PyObject *py_props)
 	return hash_props;
 }
 
-static PyObject *pyify_changed_paths(apr_hash_t *changed_paths, apr_pool_t *pool)
+PyObject *pyify_changed_paths(apr_hash_t *changed_paths, bool node_kind, apr_pool_t *pool)
 {
 	PyObject *py_changed_paths, *pyval;
 	apr_hash_index_t *idx;
@@ -315,8 +322,14 @@ static PyObject *pyify_changed_paths(apr_hash_t *changed_paths, apr_pool_t *pool
 		for (idx = apr_hash_first(pool, changed_paths); idx != NULL;
 			 idx = apr_hash_next(idx)) {
 			apr_hash_this(idx, (const void **)&key, &klen, (void **)&val);
-			pyval = Py_BuildValue("(czl)", val->action, val->copyfrom_path, 
-										 val->copyfrom_rev);
+			if (node_kind) {
+				pyval = Py_BuildValue("(czli)", val->action, val->copyfrom_path, 
+											 val->copyfrom_rev,
+											 svn_node_unknown);
+			} else {
+				pyval = Py_BuildValue("(czl)", val->action, val->copyfrom_path, 
+											 val->copyfrom_rev);
+			}
 			if (pyval == NULL)
 				return NULL;
 			PyDict_SetItemString(py_changed_paths, key, pyval);
@@ -327,13 +340,44 @@ static PyObject *pyify_changed_paths(apr_hash_t *changed_paths, apr_pool_t *pool
 	return py_changed_paths;
 }
 
+#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 6
+PyObject *pyify_changed_paths2(apr_hash_t *changed_paths, apr_pool_t *pool)
+{
+	PyObject *py_changed_paths, *pyval;
+	apr_hash_index_t *idx;
+	const char *key;
+	apr_ssize_t klen;
+	svn_log_changed_path2_t *val;
+
+	if (changed_paths == NULL) {
+		py_changed_paths = Py_None;
+		Py_INCREF(py_changed_paths);
+	} else {
+		py_changed_paths = PyDict_New();
+		for (idx = apr_hash_first(pool, changed_paths); idx != NULL;
+			 idx = apr_hash_next(idx)) {
+			apr_hash_this(idx, (const void **)&key, &klen, (void **)&val);
+			pyval = Py_BuildValue("(czli)", val->action, val->copyfrom_path, 
+										 val->copyfrom_rev, val->node_kind);
+			if (pyval == NULL)
+				return NULL;
+			PyDict_SetItemString(py_changed_paths, key, pyval);
+			Py_DECREF(pyval);
+		}
+	}
+
+	return py_changed_paths;
+}
+#endif
+
 #if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 5
 svn_error_t *py_svn_log_entry_receiver(void *baton, svn_log_entry_t *log_entry, apr_pool_t *pool)
 {
 	PyObject *revprops, *py_changed_paths, *ret;
 	PyGILState_STATE state = PyGILState_Ensure();
 
-	py_changed_paths = pyify_changed_paths(log_entry->changed_paths, pool);
+	/* FIXME: Support include node_kind */
+	py_changed_paths = pyify_changed_paths(log_entry->changed_paths, false, pool);
 	CB_CHECK_PYRETVAL(py_changed_paths);
 
 	revprops = prop_hash_to_dict(log_entry->revprops);
@@ -356,7 +400,8 @@ svn_error_t *py_svn_log_wrapper(void *baton, apr_hash_t *changed_paths, svn_revn
 	PyObject *revprops, *py_changed_paths, *ret, *obj;
 	PyGILState_STATE state = PyGILState_Ensure();
 
-	py_changed_paths = pyify_changed_paths(changed_paths, pool);
+	/*  FIXME: Support including node kind */
+	py_changed_paths = pyify_changed_paths(changed_paths, false, pool);
 	CB_CHECK_PYRETVAL(py_changed_paths);
 
 	revprops = PyDict_New();
